//------------------------------------------------------------------------------
//
//  npgl.c
//  ANTZ application based on NPE
//
//  Created by Shane Saxon on 2/19/10.
//  Copyright Saxon Digital 2010. All rights reserved.
//
//------------------------------------------------------------------------------


#include "npgl.h"

#include "../npctrl.h"


void npGLResizeScene (int width, int height);

void npCopyGPUtoCPU (void* dataRef);


//------------------------------------------------------------------------------
void npInitGL(void* dataRef) 
{
	npInitGLDraw (dataRef);

//	glShadeModel (GL_SMOOTH);
//	glClearColor (0.0f, 0.0f, 0.0f, 0.0f);
//	glClearDepth (1.0f);

	glEnable (GL_DEPTH_TEST);

//	glDepthFunc (GL_LEQUAL);
//	glHint (GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
}


//------------------------------------------------------------------------------
void npGLResizeScene (int width, int height)
{
	GLfloat ratio = 1.0;

	pData data = npDataGetDataRef();

	NPplotPtr camPlot = data->map.plot[1];
	NPcameraPtr camData = camPlot->data;

	if (height > 0)							// Prevent a divide by zero
		camData->aspectRatio = (GLfloat)width / (GLfloat)height;

	// Reset the coordinate system before modifying
	glMatrixMode (GL_PROJECTION);
	glLoadIdentity ();
	
	// Set the viewport to be the entire window
    glViewport(0, 0, width, height);

	// Set the clipping volume
	gluPerspective (camData->fov, camData->aspectRatio, 1.0f, 1000.0f);
	glMatrixMode (GL_MODELVIEW);

	npGLDrawScene (data);
}


//------------------------------------------------------------------------------
void npGLDrawScene (void* dataRef)
{
	pData data = (pData) dataRef;
	NPplotPtr camPlot = data->map.plot[1];
	NPcameraPtr camData = camPlot->data;


	npCtrlUpdateData (dataRef);			// update our input data, txt, video...
	npCtrlUpdateEngine (dataRef);		// position, physics, interactions...

	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	npDrawPlots (dataRef);				// draws terrain, graphs, pins, etc...

	glLoadIdentity();
	gluLookAt (	camPlot->translate.x, 
				camPlot->translate.y,
				camPlot->translate.z, 
				camPlot->translate.x + camPlot->rotate.x,
				camPlot->translate.y + camPlot->rotate.y,
				camPlot->translate.z + camPlot->rotate.z,
				0.0f,				//	camPlot->rotate.x,
				0.0f,				//	camPlot->rotate.y,
				1.0f );				//	camPlot->rotate.z );

	npDrawHUD (dataRef);				// draws overlays, grids, text...

	npCopyGPUtoCPU (dataRef);			// copies display to main CPU memory
}


// copy the rasterized display to the main system memory (CPU RAM or Drive)
//------------------------------------------------------------------------------
void npCopyGPUtoCPU (void* dataRef)
{

}
